'''
Created on 19.09.2010

Original found here: 
http://trac-hacks.org/wiki/YumlPreprocessorMacro

@author: dthomas (from Trac-Hacks) and Andre Netzeband

Includes a UML diagram as an image using the http://yuml.me API.
'''
from trac.wiki.macros import WikiMacroBase
from trac.db import *

import os
import urllib

revison = "$Rev$"
url = "$URL$"

class YumlPreprocessorHelper(object):
    def __init__(self, env):
        self.env = env
        self.db = env.get_db_cnx()
        self.dirname = self.getYumlCache()
        
    def getYumlCache(self):
        cdir = os.path.join(os.path.abspath(self.env.path), 'htdocs', 'cache', 'yuml')
        if not os.path.exists(cdir):
            os.makedirs(cdir)
        return os.path.join('cache','yuml')

    def getContentImage(self, content, type):
        imagecontent = type + ":" + content
        cursor = self.db.cursor()
        
        # try to find content in the cache
        cursor.execute("SELECT id FROM yumlcache WHERE content=%s", (urllib.quote(imagecontent), ))
        
        url = None
        for id in cursor:
            url = '%d.png' % id

        # if no image was found in the cache, we try now to get the image from yuml.me
        if url is None:
            url = self.getImage(content, type)
                
        if url == "error":
            return os.path.join(self.env.abs_href(),"chrome","yp","images","error.png")
                
        if url is None:        
            return None
        else:
            cdir = os.path.join(os.path.abspath(self.env.path), 'htdocs', 'cache', 'yuml', url)
        
            if not os.path.exists(cdir):
                self.deleteImage(imagecontent)
                return None
            
            return os.path.join(self.env.abs_href(),'chrome','site',self.dirname,url)
       
    def deleteImage(self, imagecontent):
        cursor = self.db.cursor()
        
        # try to find content in the cache
        cursor.execute("DELETE FROM yumlcache WHERE content=%s", (urllib.quote(imagecontent), ))
        return
     
    def getImage(self, content, type):
        url = "http://yuml.me/diagram/scruffy/" + type + "/" + urllib.quote(content)
        
        self.env.log.debug("YumlPreprocessor: Get the image from: " + url)
        
        try:
            # open picture and store it into a variable
            #mybrowser = urllib.build_opener();
            mypicturepage = urllib.urlopen(url)#mybrowser.open(url)
            infostr = "%s" % mypicturepage.info()["content-type"] 
            self.env.log.debug("YumlPreprocessor: Got from URL: " + infostr)
            if infostr != "image/png":
                return "error"
                 
            mypicture = mypicturepage.read()
        
            id = self.saveContentToDB(content, type)
        
            if id is None:
                return None
                
            dir = os.path.join(os.path.abspath(self.env.path), 'htdocs')                
            filename = os.path.join(os.path.abspath(dir), self.dirname,'%d.png' % id)

            self.env.log.debug("YumlPreprocessor: Save the image to:" + filename)
        
            file = open(filename, "wb")
            file.write(mypicture)
            file.close()
            
        except BaseException, e:
            self.env.log.debug("YumlPreprocessor - ERROR: " + e.message)
            return None
                
        return '%d.png' % id
        
    def saveContentToDB(self, content, type):
        imagecontent = type + ":" + content
        cursor = self.db.cursor()
        
        # try to find content in the cache
        cursor.execute("INSERT INTO yumlcache (content) VALUES (%s)", (urllib.quote(imagecontent),))

        # try to find content in the cache
        cursor.execute("SELECT id FROM yumlcache WHERE content=%s", (urllib.quote(imagecontent), ))
        
        number = None
        for id in cursor:
            number = id
                    
        return number

class YumlUseCaseMacro(WikiMacroBase):
    '''
    Includes a UML diagram as an image using the http://yuml.me API.
    This is intended to be used as a preprocessor.
    '''

    revision = "$Rev$"
    url = "$URL$"

    def expand_macro(self, formatter, name, content):
        '''
        Use this macro as a preprocessor macro. The content is a valid
        UseCase notation as per http://yuml.me. You may insert newlines.
        
        Example usage: (from website)
        {{{
        #!YumlUseCase
        [User]-(Login)
        [User]-(Logout) 
        (Login)<(Reminder) 
        (Login)>(Captcha)
        }}}
        '''
        yumlHelper = YumlPreprocessorHelper(self.env)
        content = content.strip().replace('\n', ', ')
        url = yumlHelper.getContentImage(content, "usecase")
        
        if url is None:
            url = 'http://yuml.me/diagram/scruffy/usecase/' + content
            
        return '<img src="'+url+'" alt="'+content+'" />'
    
    # Note that there's no need to HTML escape the returned data,
    # as the template engine (Genshi) will do it for us.

class YumlClassMacro(WikiMacroBase):
    '''
    Includes a UML diagram as an image using the http://yuml.me API.
    This is intended to be used as a preprocessor.
    '''

    revision = "$Rev$"
    url = "$URL$"

    def expand_macro(self, formatter, name, content):
        '''
        Use this macro as a preprocessor macro. The content is a valid
        class diagram notation as per http://yuml.me. You may insert newlines.
        
        Example usage: (from website)
        {{{
        #!YumlClass
        [Customer]+1->*[Order]
        [Order]++1-items >*[LineItem]
        [Order]-0..1>[PaymentMethod]
        }}}
        '''
        yumlHelper = YumlPreprocessorHelper(self.env)
        content = content.strip().replace('\n', ', ')

        url = yumlHelper.getContentImage(content, "class")
        
        if url is None:
            url = 'http://yuml.me/diagram/scruffy/class/' + content
            
        return '<img src="'+url+'" alt="'+content+'" />'
            
    # Note that there's no need to HTML escape the returned data,
    # as the template engine (Genshi) will do it for us.

class YumlActivityMacro(WikiMacroBase):
    '''
    Includes a UML diagram as an image using the http://yuml.me API.
    This is intended to be used as a preprocessor.
    '''

    revision = "$Rev$"
    url = "$URL$"

    def expand_macro(self, formatter, name, content):
        '''
        Use this macro as a preprocessor macro. The content is a valid
        activity diagram notation as per http://yuml.me. You may insert newlines.
        
        Example usage: (from website)
        {{{
        #!YumlActivity
        (start)-><d1>logged in->(Show Dashboard)->|a|->(end)
        <d1>not logged in->(Show Login)->|a|
        }}}
        '''
        yumlHelper = YumlPreprocessorHelper(self.env)
        content = content.strip().replace('\n', ', ')
        
        url = yumlHelper.getContentImage(content, "activity")
        
        if url is None:
            url = 'http://yuml.me/diagram/scruffy/activity/' + content
            
        return '<img src="'+url+'" alt="'+content+'" />'
    
    # Note that there's no need to HTML escape the returned data,
    # as the template engine (Genshi) will do it for us.
